<?php
/**
 * List of strings needed to translate for Wunderground. Never loaded, so
 * use of _ shortcut here is fine.
 *
 * @package Service_Weather
 */
// Conditions
_("Drizzle");
_("Rain");
_("Snow");
_("Snow Grains");
_("Ice Crystals");
_("Ice Pellets");
_("Hail");
_("Mist");
_("Fog");
_("Smoke");
_("Volcanic Ash");
_("Widespread Dust");
_("Sand");
_("Haze");
_("Spray");
_("Dust Whirls");
_("Sandstorm");
_("Low Drifting Snow");
_("Low Drifting Widespread Dust");
_("Low Drifting Sand");
_("Blowing Snow");
_("Blowing Widespread Dust");
_("Blowing Sand");
_("Rain Mist");
_("Rain Showers");
_("Snow Showers");
_("Ice Pellet Showers");
_("Hail Showers");
_("Small Hail Showers");
_("Thunderstorm");
_("Thunderstorms and Rain");
_("Thunderstorms and Snow");
_("Thunderstorms and Ice Pellets");
_("Thunderstorms with Hail");
_("Thunderstorms with Small Hail");
_("Freezing Drizzle");
_("Freezing Rain");
_("Freezing Fog");

_("Light Drizzle");
_("Light Rain");
_("Light Snow");
_("Light Snow Grains");
_("Light Ice Crystals");
_("Light Ice Pellets");
_("Light Hail");
_("Light Mist");
_("Light Fog");
_("Smoke");
_("Light Volcanic Ash");
_("Light Widespread Dust");
_("Light Sand");
_("Light Haze");
_("Light Spray");
_("Light Dust Whirls");
_("Light Sandstorm");
_("Light Low Drifting Snow");
_("Light Low Drifting Widespread Dust");
_("Light Low Drifting Sand");
_("Light Blowing Snow");
_("Light Blowing Widespread Dust");
_("Light Blowing Sand");
_("Light Rain Mist");
_("Light Rain Showers");
_("Light Snow Showers");
_("Light Ice Pellet Showers");
_("Light Hail Showers");
_("Light Small Hail Showers");
_("Light Thunderstorm");
_("Light Thunderstorms and Rain");
_("Light Thunderstorms and Snow");
_("Light Thunderstorms and Ice Pellets");
_("Light Thunderstorms with Hail");
_("Light Thunderstorms with Small Hail");
_("Light Freezing Drizzle");
_("Light Freezing Rain");
_("Light Freezing Fog");

_("Heavy Drizzle");
_("Heavy Rain");
_("Heavy Snow");
_("Heavy Snow Grains");
_("Heavy Ice Crystals");
_("Heavy Ice Pellets");
_("Heavy Hail");
_("Heavy Mist");
_("Heavy Fog");
_("Heavy Smoke");
_("Heavy Volcanic Ash");
_("Heavy Widespread Dust");
_("Heavy Sand");
_("Heavy Haze");
_("Heavy Spray");
_("Heavy Dust Whirls");
_("Heavy Sandstorm");
_("Heavy Low Drifting Snow");
_("Heavy Low Drifting Widespread Dust");
_("Heavy Low Drifting Sand");
_("Heavy Blowing Snow");
_("Heavy Blowing Widespread Dust");
_("Heavy Blowing Sand");
_("Heavy Rain Mist");
_("Heavy Rain Showers");
_("Heavy Snow Showers");
_("Heavy Ice Pellet Showers");
_("Heavy Hail Showers");
_("Heavy Small Hail Showers");
_("Heavy Thunderstorm");
_("Heavy Thunderstorms and Rain");
_("Heavy Thunderstorms and Snow");
_("Heavy Thunderstorms and Ice Pellets");
_("Heavy Thunderstorms with Hail");
_("Heavy Thunderstorms with Small Hail");
_("Heavy Freezing Drizzle");
_("Heavy Freezing Rain");
_("Heavy Freezing Fog");

_("Overcast");
_("Clear");
_("Partly Cloudy");
_("Mostly Cloudy");
_("Scattered Clouds");

_("Chance of Flurries");
_("Chance of Rain");
_("Chance of Freezing Rain");
_("Chance of Sleet");
_("Chance of Snow");
_("Chance of Thunderstorms");
_("Chance of a Thunderstorm");
_("Clear");
_("Cloudy");
_("Flurries");
_("Fog");
_("Haze");
_("Mostly Cloudy");
_("Mostly Sunny");
_("Partly Cloudy");
_("Partly Sunny");
_("Freezing Rain");
_("Rain");
_("Sleet");
_("Snow");
_("Sunny");
_("Thunderstorms");
_("Thunderstorm");
_("Unknown");
_("Overcast");
_("Scattered Clouds");

_("North");
_("N");
_("NNE");
_("NE");
_("ENE");
_("East");
_("E");
_("ESE");
_("SE");
_("SSE");
_("South");
_("S");
_("SSW");
_("SW");
_("WSW");
_("West");
_("W");
_("WNW");
_("NW");
_("NNW");