<?php
/**
 * List of strings needed to translate for Wwo. Never loaded, so
 * use of _ shortcut here is fine.
 *
 * @package Service_Weather
 */
_("Moderate or heavy snow in area with thunder");
_("Patchy light snow in area with thunder");
_("Moderate or heavy rain in area with thunder");
_("Patchy light rain in area with thunder");
_("Moderate or heavy showers of ice pellets");
_("Light showers of ice pellets");
_("Moderate or heavy snow showers");
_("Light snow showers");
_("Moderate or heavy sleet showers");
_("Light sleet showers");
_("Torrential rain shower");
_("Moderate or heavy rain shower");
_("Light rain shower");
_("Ice pellets");
_("Heavy snow");
_("Patchy heavy snow");
_("Moderate snow");
_("Patchy moderate snow");
_("Light snow");
_("Patchy light snow");
_("Moderate or heavy sleet");
_("Light sleet");
_("Moderate or Heavy freezing rain");
_("Light freezing rain");
_("Heavy rain");
_("Heavy rain at times");
_("Moderate rain");
_("Moderate rain at times");
_("Light rain");
_("Patchy light rain");
_("Heavy freezing drizzle");
_("Freezing drizzle");
_("Light drizzle");
_("Patchy light drizzle");
_("Freezing fog");
_("Fog");
_("Blizzard");
_("Blowing snow");
_("Thundery outbreaks nearby");
_("Patchy freezing drizzle nearby");
_("Patchy sleet nearby");
_("Patchy snow nearby");
_("Patchy rain nearby");
_("Mist");
_("Overcast");
_("Cloudy");
_("Partly Cloudy");
_("Clear");
_("Sunny");