<?php
/**
 * Exception handler for the horde/Servcie_Weather package.
 *
 * Copyright 2011-2017 Horde LLC (http://www.horde.org/)
 *
 * @author   Michael J Rubinsky <mrubinsk@horde.org>
 * @category Horde
 * @license  http://www.horde.org/licenses/bsd BSD
 * @package  Service_Weather
 */
class Horde_Service_Weather_Exception_InvalidProperty extends Horde_Service_Weather_Exception
{
}